'use strict';

import fs from 'fs';
import path from 'path';
import Promise from 'bluebird';
import { runCmd } from '../util';
import chalk from 'chalk';
import { Base } from 'yeoman-generator';
import { genBase } from '../generator-base';
import insight from '../insight-init';
import { exec } from 'child_process';
import babelStream from 'gulp-babel';
import beaufityStream from 'gulp-beautify';
import tap from 'gulp-tap';
import filter from 'gulp-filter';
import eslint from 'gulp-eslint';
import semver from 'semver';

export class Generator extends Base {
  constructor(...args) {
    super(...args);

    this.argument('name', { type: String, required: false });

    this.option('skip-install', {
      desc: 'Do not install dependencies',
      type: Boolean,
      defaults: false
    });

    this.option('skip-config', {
      desc: 'Always use existing .yo-rc.json',
      type: Boolean,
      defaults: false
    });

    this.option('app-suffix', {
      desc: 'Allow a custom suffix to be added to the module name',
      type: String,
      defaults: 'App'
    });

    this.option('dev-port', {
      desc: 'Port to use for the development HTTP server',
      type: String,
      defaults: '9000'
    });

    this.option('debug-port', {
      desc: 'Port to use for the server debugger',
      type: String,
      defaults: '5858'
    });

    this.option('prod-port', {
      desc: 'Port to use for the production HTTP Server',
      type: String,
      defaults: '8080'
    });
  }

  get initializing() {
    return {
      init: function () {
        this.config.set('generatorVersion', this.rootGeneratorVersion());
        this.filters = {};

        // init shared generator properies and methods
        const genBasePromise = genBase(this);
        let promises = [genBasePromise];

        if (process.env.CI) {
          insight.optOut = true;
        } else if (insight.optOut === undefined) {
          promises.push(new Promise((resolve, reject) => {
            insight.askPermission(null, (err, optIn) => {
              if (err) return reject(err);
              else return resolve(optIn);
            });
          }));
        }

        insight.track('generator', this.rootGeneratorVersion());
        this.nodeVersion = semver.clean(process.version);
        this.semver = semver;
        insight.track('node', this.nodeVersion);
        insight.track('platform', process.platform);

        const npmVersionPromise = runCmd('npm --version').then(stdout => {
          this.npmVersion = stdout.toString().trim();
          return insight.track('npm', this.npmVersion);
        });
        promises.push(npmVersionPromise);

        return Promise.all(promises);
      },
      info: function () {
        this.log(this.yoWelcome);
        this.log('Angular Fullstack v' + this.rootGeneratorVersion() + '\n');
        this.log('Out of the box I create an AngularJS app with an Express server.\n');
      },
      checkForConfig: function () {
        var existingFilters = this.config.get('filters');

        if (!existingFilters) return;

        let promise = this.options['skip-config']
          ? Promise.resolve({ skipConfig: true })
          : this.prompt([{
            type: 'confirm',
            name: 'skipConfig',
            message: 'Existing .yo-rc configuration found, would you like to use it?',
            default: true,
          }]);

        return promise.then(answers => {
          this.skipConfig = answers.skipConfig;

          if (this.skipConfig) {
            insight.track('skipConfig', 'true');
            this.filters = existingFilters;

            this.scriptExt = this.filters.ts ? 'ts' : 'js';
            this.templateExt = this.filters.pug ? 'pug' : 'html';
            this.styleExt = this.filters.sass ? 'scss' :
              this.filters.less ? 'less' :
                this.filters.stylus ? 'styl' :
                  'css';
          } else {
            insight.track('skipConfig', 'false');
            this.filters = {};
            this.forceConfig = true;
            this.config.set('filters', this.filters);
            this.config.forceSave();
          }
        });
      },
      assignPorts() {
        this.devPort = this.options['dev-port'];
        this.debugPort = this.options['debug-port'];
        this.prodPort = this.options['prod-port'];
      }
    };
  }

  get prompting() {
    return {
    }
  }

  get configuring() {
    return {
    }
  }

  get default() {
    return {};
  }

  get writing() {
    return {
      generateProject: function() {       
        let self = this;
        this.sourceRoot(path.join(__dirname, '../../templates'));
        this.processDirectory('.', '.');
      }     
    };

  }

  install() {
    if (!this.options['skip-install']) {
      let yarnCheckCommand;
      if (process.platform === 'win32') {
        yarnCheckCommand = 'yarn --version >nul 2>&1';
      } else {
        yarnCheckCommand = 'type yarn &> /dev/null';
      }
      exec(yarnCheckCommand, (error, stdout, stderr) => {
        return this.spawnCommand((!error) ? 'yarn' : 'npm', ['install']);
      });
    }
  }

  get end() {
    return {};
  }
}

module.exports = Generator;
